#include "cglib/matrix4x4.h"
#include <exception>

Point3 Matrix4x4::operator*(const Point3 &p) const
{
	float x = m[0][0] * p.x + m[0][1] * p.y + m[0][2] * p.z + m[0][3];
	float y = m[1][0] * p.x + m[1][1] * p.y + m[1][2] * p.z + m[1][3];
	float z = m[2][0] * p.x + m[2][1] * p.y + m[2][2] * p.z + m[2][3];
	float w = m[3][0] * p.x + m[3][1] * p.y + m[3][2] * p.z + m[3][3];

	if (w == 1.0f) 
		return Point3(x,y,z);
	else
	{
		float wi = 1.0f / w;
		return Point3(x * wi, y * wi, z * wi);
	}
}

Vector3 Matrix4x4::operator*(const Vector3 &v) const
{
	float x = m[0][0] * v.x + m[0][1] * v.y + m[0][2] * v.z;
	float y = m[1][0] * v.x + m[1][1] * v.y + m[1][2] * v.z;
	float z = m[2][0] * v.x + m[2][1] * v.y + m[2][2] * v.z;
	return Vector3(x,y,z);
}	

Matrix4x4 Matrix4x4::look_at(Point3 eye, Point3 at, Vector3 up)
{
	Vector3 z = normalize(eye - at);
	up = normalize(up);
	Vector3 x = cross(up, z);
	Vector3 y = cross(z, x);

	return Matrix4x4(
		x[0], y[0], z[0], eye[0],
		x[1], y[1], z[1], eye[1],
		x[2], y[2], z[2], eye[2],
		0.0f, 0.0f, 0.0f, 1.0f);
}

Matrix4x4 Matrix4x4::orthographic_projection(float left, float right, float bottom, float top, float near, float far)
{
	return Matrix4x4(
		2/(right - left), 0, 0, -(right+left)/(right-left),
		0, 2/(top - bottom), 0, -(top+bottom)/(top-bottom),
		0, 0, -2/(far - near), -(far+near)/(far-near),
		0, 0, 0, 1);
}

Matrix4x4 Matrix4x4::perspective_projection(float left, float right, float bottom, float top, float near, float far)
{
	return Matrix4x4(
		2*near/(right-left), 0, (right+left)/(right-left), 0,
		0, 2*near/(top-bottom), (top+bottom)/(top-bottom), 0,
		0, 0, -(far+near)/(far-near), -2*far*near/(far-near),
		0, 0, -1, 0);
}

Matrix4x4 Matrix4x4::perspective_projection(float fovy, float aspect, float near, float far)
{
	float top = (float)(near * tanf(fovy * M_PI / 180.0f / 2.0f));
	float bottom = -top;
	float right = aspect * top;
	float left = -right;
	return perspective_projection(left, right, bottom, top, near, far);
}