// Arcball camera by Eugene Hsu
// Based on 6.839 sample code for rotation code.
// Extended to handle translation (MIDDLE) and scale (RIGHT)

// -*-c++-*-
#ifndef __ARCBALL_CAMERA_H__
#define __ARCBALL_CAMERA_H__

#include "cglib/vector3.h"
#include "cglib/point3.h"
#include "cglib/matrix4x4.h"

class ArcballCamera
{
public:

    ArcballCamera();
    
    typedef enum { NONE, LEFT, MIDDLE, RIGHT } Button;

    // You must call all of the Set*() functions before you use this!
    // I didn't put it into the constructor because it's inconvenient
    // to initialize stuff in my opengl application.
    
    void SetDimensions(int w, int h);
    void SetViewport(int x, int y, int w, int h);
    void SetPerspective(float fovy);

    // Call from whatever UI toolkit
    void MouseClick(Button button, int x, int y);
    void MouseDrag(int x, int y);
    void MouseRelease(int x, int y);

    // Apply viewport, perspective, and modeling
    // use these instead of 
    void ApplyViewport() const;
    void ApplyPerspective() const;
    void ApplyModelview();

    // Set for relevant vars
    void SetCenter(const Point3 & center);
    void SetRotation(const Matrix4x4 & rotation);
    void SetDistance(const float distance);

    // Get for relevant vars
    Point3 GetCenter() const { return mCurrentCenter; }
    Matrix4x4 GetRotation() const { return mCurrentRot; }
    float GetDistance() const { return mCurrentDistance; }   

    // States 
    int     mDimensions[2];
    int     mStartClick[2];
    Button  mButtonState;

    // For rotation
    Matrix4x4 mStartRot;
    Matrix4x4 mCurrentRot;

    // For translation
    float mPerspective[2];
    int   mViewport[4];
    Point3 mStartCenter;
    Point3 mCurrentCenter;

    // For zoom
    float mStartDistance;
    float mCurrentDistance;

    void ArcBallRotation(int x, int y);
    void PlaneTranslation(int x, int y);
    void DistanceZoom(int x, int y);
};

#endif
