#ifndef __GLUT_WINDOW_H__
#define __GLUT_WINDOW_H__

#include "windows.h"
#include <string>
#include <map>
#include <GL/glew.h>
#include <GL/glut.h>
#include "cglib/arcball_camera.h"

using namespace std;

struct NormalKey
{
	unsigned char key;
	int modifiers;

	NormalKey()
	{
		key = 0;
		modifiers = 0;
	}

	NormalKey(unsigned char _key, int _modifiers)
	{
		key = _key;
		modifiers = _modifiers;
	}		
};

struct NormalKeyOrdering
{
	bool operator()(const NormalKey a, const NormalKey b) const
	{
		return (a.key < b.key) || (a.key == b.key && a.modifiers < b.modifiers);
	}
};

struct SpecialKey
{
	int key;
	int modifiers;

	SpecialKey()
	{
		key = 0;
		modifiers = 0;
	}

	SpecialKey(int _key, int _modifiers)
	{
		key = _key;
		modifiers = _modifiers;
	}
};

struct SpecialKeyOrdering
{
	bool operator()(const SpecialKey a, const SpecialKey b) const
	{
		return (a.key < b.key) || (a.key == b.key && a.modifiers < b.modifiers);
	}
};

typedef void (*NormalKeyHandler)(unsigned char, int, int);
typedef void (*SpecialKeyHandler)(int, int, int);
typedef map<NormalKey, NormalKeyHandler, NormalKeyOrdering> NormalKeyHandlerMap;
typedef map<SpecialKey, SpecialKeyHandler, SpecialKeyOrdering> SpecialKeyHandlerMap;

class GlutWindow
{
protected:
	static GlutWindow *GlutWindow_instance;

	static void StaticDisplayFunc();
	static void StaticReshapeFunc(int width, int height);
	static void StaticIdleFunc();	
	static void StaticMouseFunc(int button, int state, int x, int y);
	static void StaticMotionFunc(int x, int y);
	static void StaticKeyboardFunc(unsigned char key, int x, int y);
	static void StaticSpecialFunc(int key, int x, int y);
	
	GlutWindow()
	{
		GlutWindow_instance = this;
	}

public:
	virtual void DisplayFunc();
	virtual void ReshapeFunc(int width, int height);
	virtual void IdleFunc();
	virtual void MouseFunc(int button, int state, int x, int y);
	virtual void MotionFunc(int x, int y);
	virtual void KeyboardFunc(unsigned char key, int x, int y);
	virtual void SpecialFunc(int key, int x, int y);

	NormalKeyHandlerMap normalKeyHandlers;
	SpecialKeyHandlerMap specialKeyHandlers;

	int windowWidth, windowHeight;
	string windowTitle;

	virtual void Initialize();
	virtual void InitDisplay();
	virtual void Draw();

	virtual void Run(int *argc, char **argv);

	virtual GlutWindow *GetGlutWindowInstance();
};

#endif