#ifndef _MESH_MATERIAL_MAPPING_H_
#define _MESH_MATERIAL_MAPPING_H_

#include "cglib/object.h"
#include <vector>

//! Represent a mapping from a mesh's faces to indices of the material of that face.
class MeshMaterialMapping : public Object
{
public:
    //! A constructor that does nothing.
    MeshMaterialMapping ();
    
    //! A destructor that does nothing.
    virtual ~MeshMaterialMapping ();
    
    //! Return the number of faces of the associated mesh.
    /*!
        The value returned by the method is equal to the
        number of material indices stored.
     */
    int get_face_count() const;
    
    //! Return the index of the material of the face with the given index.
    int get_face_material_index(int face_index) const;
    
    //! Add a new material index.
    /*!
        The material index added will be associated with
        the face whose index is the number of 
        material indices stored at the time of addition.
        
        For example, the first material index added 
        with the associated with Face 0. The second material
        index added will be associated with Face 1. And so on.
     */
	void append_face_material_index(int material_index);
	
	//! Accept a visitor.
	virtual void accept(Visitor *visitor);
	
private:
	std::vector<int> mappings;
};

#endif /* _MESH_MATERIAL_MAPPING_H_ */
