#include <windows.h>
#include <GL/glew.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <IL/il.h>
#include <IL/ilu.h>
#include <IL/ilut.h>
#include "cglib/arcball_window.h"
#include "cglib/parser.h"
#include "cglib/path.h"
#include "cglib/mesh.h"
#include "cglib/reference.h"
#include <iostream>
#include <string>

extern Mesh *parse_obj_file(const char *);
static Reference<Mesh> mesh = NULL;
static std::string file_name;

void init_scene()
{
	std::cout << "Parsing..." << std::endl;
	Parser parser;
	try
	{
		mesh = dynamic_cast<Mesh *>(parse_obj_file(file_name.c_str()));
		std::cout << "Parsing done." << std::endl;
	}
	catch (exception &e)
	{
		std::cout << e.what() << std::endl;
	}
}

class MyApp : public ArcballWindow
{
public:
	MyApp() : ArcballWindow()
	{
		windowWidth = 512;
		windowHeight = 512;
		windowTitle = "Problem 4 Test";
	}

	virtual void InitDisplay()
	{
		camera.SetCenter(Point3(0,0,0));
		camera.SetDistance(5);
		camera.SetRotation(Matrix4x4(
			1,0,0,0,
			0,1,0,0,
			0,0,1,0,
			0,0,0,1));
		camera.SetPerspective(60.0f);
		camera.SetViewport(0,0,windowWidth,windowHeight);

		glShadeModel(GL_SMOOTH);
		glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
		glClearDepth(1.0f);
		glEnable(GL_DEPTH_TEST);
		glDepthFunc(GL_LESS);
		glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		glEnable(GL_CULL_FACE);

		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);

		float light_diffuse_color[4] = {1.0f, 1.0f, 1.0f, 1.0f};
		float light_specular_color[4] = {1.0f, 1.0f, 1.0f, 1.0f};
		float light_ambient_color[4] = {1.0f, 1.0f, 1.0f, 1.0f};		
		float light_position[4] = {0.0f, 0.0f, 20.0f, 1.0f};

		glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse_color);
		glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular_color);
		glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient_color);		
		glLightfv(GL_LIGHT0, GL_POSITION, light_position);

		float material_diffuse_color[4] = {0.5f, 0.9f, 0.9f, 1.0f};
		float material_specular_color[4] = {1.0f, 1.0f, 1.0f, 1.0f};
		float material_ambient_color[4] = {0.1f, 0.1f, 0.1f, 1.0f};
		float material_shininess = 20.0f;

		glMaterialfv(GL_FRONT, GL_DIFFUSE, material_diffuse_color);
		glMaterialfv(GL_FRONT, GL_SPECULAR, material_specular_color);
		glMaterialfv(GL_FRONT, GL_AMBIENT, material_ambient_color);
		glMaterialf(GL_FRONT, GL_SHININESS, material_shininess);

		init_scene();
	}

	virtual void ReshapeFunc(int x, int y)
	{
		camera.SetViewport(0,0,x,y);
		glutPostRedisplay();
	}

	virtual void DisplayFunc()
	{		
		glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		camera.ApplyViewport();
		camera.ApplyPerspective();

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		camera.ApplyModelview();

		Draw();

		glutSwapBuffers();
	}

	virtual void Draw()
	{	
		FOR(face_index, mesh->get_face_count())
		{
			glBegin(GL_POLYGON);

			FOR(vertex_index, mesh->get_face_vertex_count(face_index))
			{
				Point3 position = mesh->get_face_vertex_position(face_index, vertex_index);
				Vector3 normal = mesh->get_face_vertex_normal(face_index, vertex_index);

				glNormal3f(normal.x, normal.y, normal.z);
				glVertex3f(position.x, position.y, position.z);
			}

			glEnd();
		}
	}
};

MyApp *app;

int main(int argc, char **argv)
{
	ilInit();
	iluInit();
	ilutInit();

	if (argc < 2)
	{
		std::cout << "Usage: problem4_test OBJ_FILE_NAME" << std::endl;
	}
	else
	{
		file_name = argv[1];
		file_name = Path::v()->get_absolute_path(file_name);
		std::cout << "File name: " << file_name << std::endl;

		app = new MyApp();
		app->Run(&argc, argv);
		return 0;
	}
}