#include <windows.h>
#include <GL/glew.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <IL/il.h>
#include <IL/ilu.h>
#include <IL/ilut.h>
#include "cglib/arcball_window.h"
#include "cglib/parser.h"
#include "cglib/path.h"
#include "cglib/mesh_node.h"
#include <iostream>
#include <string>

extern void render(MeshNode *node);

static Reference<MeshNode> node = NULL;
static std::string file_name;

void init_scene()
{
	std::cout << "Parsing..." << std::endl;
	Parser parser;
	try
	{
		node = dynamic_cast<MeshNode *>(parser.parse(file_name));
		std::cout << "Parsing done." << std::endl;
	}
	catch (exception &e)
	{
		std::cout << e.what() << std::endl;
	}
}

class MyApp : public ArcballWindow
{
public:
	MyApp() : ArcballWindow()
	{
		windowWidth = 512;
		windowHeight = 512;
		windowTitle = "Problem 5 Test";
	}

	virtual void InitDisplay()
	{
		camera.SetCenter(Point3(0,10,0));
		camera.SetDistance(20);
		camera.SetRotation(Matrix4x4(
			1,0,0,0,
			0,1,0,0,
			0,0,1,0,
			0,0,0,1));
		camera.SetPerspective(60.0f);
		camera.SetViewport(0,0,windowWidth,windowHeight);

		glShadeModel(GL_SMOOTH);
		glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
		glClearDepth(1.0f);
		glEnable(GL_DEPTH_TEST);
		glDepthFunc(GL_LESS);
		glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		glEnable(GL_CULL_FACE);

		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);

		float diffuse_color[4] = {1.0f, 1.0f, 1.0f, 1.0f};
		float specular_color[4] = {1.0f, 1.0f, 1.0f, 1.0f};
		float ambient_color[4] = {0.5f, 0.5f, 0.5f, 1.0f};
		float shininess = 20.0f;
		float light_position[4] = {0.0f, 0.0f, 20.0f, 1.0f};

		glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse_color);
		glLightfv(GL_LIGHT0, GL_SPECULAR, specular_color);
		glLightfv(GL_LIGHT0, GL_AMBIENT, ambient_color);
		glLightf(GL_LIGHT0, GL_SHININESS, shininess);
		glLightfv(GL_LIGHT0, GL_POSITION, light_position);

		init_scene();
	}

	virtual void ReshapeFunc(int x, int y)
	{
		camera.SetViewport(0,0,x,y);
		glutPostRedisplay();
	}

	virtual void DisplayFunc()
	{		
		glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		camera.ApplyViewport();
		camera.ApplyPerspective();

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		camera.ApplyModelview();

		Draw();

		glutSwapBuffers();
	}

	virtual void Draw()
	{	
		render(node.get_pointer());		
	}
};

MyApp *app;

int main(int argc, char **argv)
{
	ilInit();
	iluInit();
	ilutInit();

	if (argc < 2)
	{
		std::cout << "Usage: view_ezt MODEL_FILE_NAME" << std::endl;
	}
	else
	{
		file_name = argv[1];
		file_name = Path::v()->get_absolute_path(file_name);
		std::cout << "File name: " << file_name << std::endl;

		app = new MyApp();
		app->Run(&argc, argv);
		return 0;
	}
}